package jdk.management.resource.internal.inst;

import sun.misc.JavaIOFileDescriptorAccess;
import sun.misc.SharedSecrets;
import java.io.IOException;
import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.FileNotFoundException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.io.FileDescriptor;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.io.RandomAccessFile")
public final class RandomAccessFileRMHooks
{
    private FileDescriptor fd;
    private final String path;
    private final Object closeLock;
    private volatile boolean closed;
    
    public RandomAccessFileRMHooks() {
        this.path = null;
        this.closeLock = new Object();
        this.closed = false;
    }
    
    @InstrumentationMethod
    private void open(String s, int n) throws FileNotFoundException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_OPEN_GROUP.getApprover(this);
        long request = 0L;
        long request2;
        try {
            request2 = approver.request(1L, of);
            if (request2 < 1L) {
                throw new FileNotFoundException(s + ": resource limited: too many open files");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            FileNotFoundException ex = new FileNotFoundException(s + ": resource limited: too many open files");
            ex.initCause(cause);
            throw ex;
        }
        ResourceRequest approver2 = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd);
        int n2 = 0;
        try {
            try {
                request = approver2.request(1L, of);
                if (request < 1L) {
                    throw new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                }
            }
            catch (ResourceRequestDeniedException cause2) {
                FileNotFoundException ex2 = new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                ex2.initCause(cause2);
                throw ex2;
            }
            this.open(s, n);
            n2 = 1;
        }
        finally {
            approver2.request(-(request - (long)n2), of);
            approver.request(-(request2 - (long)n2), of);
        }
    }
    
    @InstrumentationMethod
    public int read() throws IOException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int read = -1;
        try {
            read = this.read();
        }
        finally {
            approver.request(-(request - (long)((read == -1) ? false : true)), of);
        }
        return read;
    }
    
    @InstrumentationMethod
    public int read(byte[] array) throws IOException {
        if (array == null) {
            return this.read(array);
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        int length = array.length;
        long max;
        try {
            max = Math.max(approver.request((long)length, of), 0L);
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n = 0;
        int n2;
        try {
            if (max < (long)length) {
                approver.request(-max, of);
                n2 = this.read(array, 0, array.length);
                n = Math.max(n2, 0);
            }
            else {
                n2 = this.read(array);
                n = Math.max(n2, 0);
            }
        }
        finally {
            approver.request(-(max - (long)n), of);
        }
        return n2;
    }
    
    @InstrumentationMethod
    public int read(byte[] array, int n, int min) throws IOException {
        if (min < 0) {
            return this.read(array, n, min);
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        long max;
        try {
            max = Math.max(approver.request((long)min, of), 0L);
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        min = Math.min(min, (int)max);
        int max2 = 0;
        int read = 0;
        try {
            read = this.read(array, n, min);
            max2 = Math.max(read, 0);
        }
        finally {
            approver.request(-(max - (long)max2), of);
        }
        return read;
    }
    
    @InstrumentationMethod
    public void write(int n) throws IOException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n2 = 0;
        try {
            this.write(n);
            n2 = 1;
        }
        finally {
            approver.request(-(request - (long)n2), of);
        }
    }
    
    @InstrumentationMethod
    public void write(byte[] array) throws IOException {
        if (array == null) {
            this.write(array);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        int length = array.length;
        long request;
        try {
            request = approver.request((long)length, of);
            if (request < (long)length) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n = 0;
        try {
            this.write(array);
            n = length;
        }
        finally {
            approver.request(-(request - (long)n), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public void write(byte[] array, int n, int n2) throws IOException {
        if (n2 < 0) {
            this.write(array, n, n2);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request((long)n2, of);
            if (request < (long)n2) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n3 = 0;
        try {
            this.write(array, n, n2);
            n3 = n2;
        }
        finally {
            approver.request(-(request - (long)n3), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public final void writeBytes(String s) throws IOException {
        if (s == null) {
            this.writeBytes(s);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        int length = s.length();
        long request;
        try {
            request = approver.request((long)length, of);
            if (request < (long)length) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n = 0;
        try {
            this.writeBytes(s);
            n = length;
        }
        finally {
            approver.request(-(request - (long)n), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public final void writeChars(String s) throws IOException {
        if (s == null) {
            this.writeChars(s);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        int n = 2 * s.length();
        long request;
        try {
            request = approver.request((long)n, of);
            if (request < (long)n) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n2 = 0;
        try {
            this.writeChars(s);
            n2 = n;
        }
        finally {
            approver.request(-(request - (long)n2), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.closed) {
                return;
            }
        }
        JavaIOFileDescriptorAccess javaIOFileDescriptorAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        long handle;
        try {
            handle = javaIOFileDescriptorAccess.getHandle(this.fd);
            if (handle == -1L) {
                handle = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
        }
        catch (UnsupportedOperationException ex) {
            handle = (long)javaIOFileDescriptorAccess.get(this.fd);
        }
        try {
            this.close();
            long handle2;
            try {
                handle2 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle2 == -1L) {
                    handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex2) {
                handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of = ResourceIdImpl.of(this.path);
            if (handle2 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of);
        }
        finally {
            long handle3;
            try {
                handle3 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle3 == -1L) {
                    handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex3) {
                handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of2 = ResourceIdImpl.of(this.path);
            if (handle3 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of2);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of2);
        }
    }
}
